@@ -75,16 +75,16 @@ macro_rules! fold_der_defined_m(
 /// Internal parser, do not use directly
 #[doc(hidden)]
 #[macro_export]
-macro_rules! parse_der_defined_m(
+macro_rules! parse_ber_defined_m(
     ($i:expr, $tag:expr, $($args:tt)*) => (
         {
-            use $crate::der_read_element_header;
+            use $crate::ber::ber_read_element_header;
             do_parse!(
                 $i,
-                hdr:     der_read_element_header >>
-                         error_if!(hdr.class != 0b00, ErrorKind::Custom($crate::DER_CLASS_ERROR)) >>
-                         error_if!(hdr.structured != 0b1, ErrorKind::Custom($crate::DER_STRUCT_ERROR)) >>
-                         error_if!(hdr.tag != $tag, ErrorKind::Custom($crate::DER_TAG_ERROR)) >>
+                hdr:     ber_read_element_header >>
+                         error_if!(hdr.class != 0b00, ErrorKind::Custom($crate::error::BER_CLASS_ERROR)) >>
+                         error_if!(hdr.structured != 0b1, ErrorKind::Custom($crate::error::BER_STRUCT_ERROR)) >>
+                         error_if!(hdr.tag != $tag, ErrorKind::Custom($crate::error::BER_TAG_ERROR)) >>
                 content: flat_take!(hdr.len as usize, fold_der_defined_m!( $($args)* )) >>
                 (hdr,content)
             )
@@ -100,18 +100,18 @@ macro_rules! parse_der_defined_m(
 /// Similar to [`parse_der_sequence_defined`](macro.parse_der_sequence_defined.html), but not using `fold`.
 /// This allow using macros.
 ///
-/// ```rust,no_run
+/// ```rust
 /// # #[macro_use] extern crate nom;
 /// # #[macro_use] extern crate rusticata_macros;
 /// # #[macro_use] extern crate der_parser;
-/// use der_parser::*;
+/// use der_parser::ber::*;
 /// use nom::{IResult,Err,ErrorKind};
 ///
 /// # fn main() {
-/// fn localparse_seq(i:&[u8]) -> IResult<&[u8],DerObject> {
+/// fn localparse_seq(i:&[u8]) -> IResult<&[u8],BerObject> {
 ///     parse_der_sequence_defined_m!(i,
-///         parse_der_integer >>
-///         call!(parse_der_integer)
+///         parse_ber_integer >>
+///         call!(parse_ber_integer)
 ///     )
 /// }
 /// let empty = &b""[..];
@@ -119,22 +119,22 @@ macro_rules! parse_der_defined_m(
 ///               0x02, 0x03, 0x01, 0x00, 0x01,
 ///               0x02, 0x03, 0x01, 0x00, 0x00,
 /// ];
-/// let expected  = DerObject::from_obj(DerObjectContent::Sequence(vec![
-///     DerObject::from_int_slice(b"\x01\x00\x01"),
-///     DerObject::from_int_slice(b"\x01\x00\x00"),
-/// ]));
+/// let expected  = BerObject::from_seq(vec![
+///     BerObject::from_int_slice(b"\x01\x00\x01"),
+///     BerObject::from_int_slice(b"\x01\x00\x00"),
+/// ]);
 /// assert_eq!(localparse_seq(&bytes), Ok((empty, expected)));
 /// # }
 /// ```
 #[macro_export]
 macro_rules! parse_der_sequence_defined_m(
-    ($i:expr, $($args:tt)*) => (
+    ($i:expr, $($args:tt)*) => ({
         map!(
             $i,
-            parse_der_defined_m!(0x10, $($args)*),
-            |(hdr,o)| $crate::DerObject::from_header_and_content(hdr,$crate::DerObjectContent::Sequence(o))
+            parse_ber_defined_m!($crate::ber::BerTag::Sequence, $($args)*),
+            |(hdr,o)| $crate::ber::BerObject::from_header_and_content(hdr,$crate::ber::BerObjectContent::Sequence(o))
         )
-    );
+    });
 );
 
 /// Parse a set of DER elements (macro version)
@@ -145,18 +145,18 @@ macro_rules! parse_der_sequence_defined_m(
 /// Similar to [`parse_der_set_defined`](macro.parse_der_set_defined.html), but not using `fold`.
 /// This allow using macros.
 ///
-/// ```rust,no_run
+/// ```rust
 /// # #[macro_use] extern crate nom;
 /// # #[macro_use] extern crate rusticata_macros;
 /// # #[macro_use] extern crate der_parser;
-/// use der_parser::*;
+/// use der_parser::ber::*;
 /// use nom::{IResult,Err,ErrorKind};
 ///
 /// # fn main() {
-/// fn localparse_set(i:&[u8]) -> IResult<&[u8],DerObject> {
+/// fn localparse_set(i:&[u8]) -> IResult<&[u8],BerObject> {
 ///     parse_der_set_defined_m!(i,
-///         parse_der_integer >>
-///         call!(parse_der_integer)
+///         parse_ber_integer >>
+///         call!(parse_ber_integer)
 ///     )
 /// }
 /// let empty = &b""[..];
@@ -164,22 +164,22 @@ macro_rules! parse_der_sequence_defined_m(
 ///               0x02, 0x03, 0x01, 0x00, 0x01,
 ///               0x02, 0x03, 0x01, 0x00, 0x00,
 /// ];
-/// let expected  = DerObject::from_obj(DerObjectContent::Set(vec![
-///     DerObject::from_int_slice(b"\x01\x00\x01"),
-///     DerObject::from_int_slice(b"\x01\x00\x00"),
-/// ]));
+/// let expected  = BerObject::from_set(vec![
+///     BerObject::from_int_slice(b"\x01\x00\x01"),
+///     BerObject::from_int_slice(b"\x01\x00\x00"),
+/// ]);
 /// assert_eq!(localparse_set(&bytes), Ok((empty, expected)));
 /// # }
 /// ```
 #[macro_export]
 macro_rules! parse_der_set_defined_m(
-    ($i:expr, $($args:tt)*) => (
+    ($i:expr, $($args:tt)*) => ({
         map!(
             $i,
-            parse_der_defined_m!(0x11, $($args)*),
-            |(hdr,o)| $crate::DerObject::from_header_and_content(hdr,$crate::DerObjectContent::Set(o))
+            parse_ber_defined_m!($crate::ber::BerTag::Set, $($args)*),
+            |(hdr,o)| $crate::ber::BerObject::from_header_and_content(hdr,$crate::ber::BerObjectContent::Set(o))
         )
-    );
+    });
 );
 
 
@@ -211,15 +211,15 @@ macro_rules! fold_parsers(
 macro_rules! parse_der_defined(
     ($i:expr, $ty:expr, $($args:tt)*) => (
         {
-            use $crate::der_read_element_header;
+            use $crate::ber::ber_read_element_header;
             use nom::ErrorKind;
             let res =
             do_parse!(
                 $i,
-                hdr:     der_read_element_header >>
-                         error_if!(hdr.class != 0b00, ErrorKind::Custom($crate::DER_CLASS_ERROR)) >>
-                         error_if!(hdr.structured != 0b1, ErrorKind::Custom($crate::DER_STRUCT_ERROR)) >>
-                         error_if!(hdr.tag != $ty, ErrorKind::Custom($crate::DER_TAG_ERROR)) >>
+                hdr:     ber_read_element_header >>
+                         error_if!(hdr.class != 0b00, ErrorKind::Custom($crate::error::BER_CLASS_ERROR)) >>
+                         error_if!(hdr.structured != 0b1, ErrorKind::Custom($crate::error::BER_STRUCT_ERROR)) >>
+                         error_if!(hdr.tag != $ty, ErrorKind::Custom($crate::error::BER_TAG_ERROR)) >>
                 content: take!(hdr.len) >>
                 (hdr,content)
             );
@@ -227,7 +227,7 @@ macro_rules! parse_der_defined(
                 Ok((_rem,o)) => {
                     match fold_parsers!(o.1, $($args)* ) {
                         Ok((rem,v)) => {
-                            if rem.len() != 0 { Err(::nom::Err::Error(error_position!($i, ErrorKind::Custom(DER_OBJ_TOOSHORT)))) }
+                            if rem.len() != 0 { Err(::nom::Err::Error(error_position!($i, ErrorKind::Custom($crate::error::BER_OBJ_TOOSHORT)))) }
                             else { Ok((_rem,(o.0,v))) }
                         },
                         Err(e)      => Err(e)
@@ -248,18 +248,18 @@ macro_rules! parse_der_defined(
 /// `fold` internally.
 /// Because of that, macros cannot be used as subparsers.
 ///
-/// ```rust,no_run
+/// ```rust
 /// # #[macro_use] extern crate nom;
 /// # #[macro_use] extern crate rusticata_macros;
 /// # #[macro_use] extern crate der_parser;
-/// use der_parser::*;
+/// use der_parser::ber::*;
 /// use nom::{IResult,Err,ErrorKind};
 ///
 /// # fn main() {
-/// fn localparse_seq(i:&[u8]) -> IResult<&[u8],DerObject> {
+/// fn localparse_seq(i:&[u8]) -> IResult<&[u8],BerObject> {
 ///     parse_der_sequence_defined!(i,
-///         parse_der_integer,
-///         parse_der_integer
+///         parse_ber_integer,
+///         parse_ber_integer
 ///     )
 /// }
 /// let empty = &b""[..];
@@ -267,10 +267,10 @@ macro_rules! parse_der_defined(
 ///               0x02, 0x03, 0x01, 0x00, 0x01,
 ///               0x02, 0x03, 0x01, 0x00, 0x00,
 /// ];
-/// let expected  = DerObject::from_obj(DerObjectContent::Sequence(vec![
-///     DerObject::from_int_slice(b"\x01\x00\x01"),
-///     DerObject::from_int_slice(b"\x01\x00\x00"),
-/// ]));
+/// let expected  = BerObject::from_seq(vec![
+///     BerObject::from_int_slice(b"\x01\x00\x01"),
+///     BerObject::from_int_slice(b"\x01\x00\x00"),
+/// ]);
 /// assert_eq!(localparse_seq(&bytes), Ok((empty, expected)));
 /// # }
 /// ```
@@ -279,8 +279,8 @@ macro_rules! parse_der_sequence_defined(
     ($i:expr, $($args:tt)*) => (
         map!(
             $i,
-            parse_der_defined!(0x10, $($args)*),
-            |(hdr,o)| $crate::DerObject::from_header_and_content(hdr,$crate::DerObjectContent::Sequence(o))
+            parse_der_defined!($crate::ber::BerTag::Sequence, $($args)*),
+            |(hdr,o)| $crate::ber::BerObject::from_header_and_content(hdr,$crate::ber::BerObjectContent::Sequence(o))
         )
     );
 );
@@ -294,29 +294,29 @@ macro_rules! parse_der_sequence_defined(
 /// `fold` internally.
 /// Because of that, macros cannot be used as subparsers.
 ///
-/// ```rust,no_run
+/// ```rust
 /// # #[macro_use] extern crate nom;
 /// # #[macro_use] extern crate rusticata_macros;
 /// # #[macro_use] extern crate der_parser;
-/// use der_parser::*;
+/// use der_parser::ber::*;
 /// use nom::{IResult,Err,ErrorKind};
 ///
 /// # fn main() {
-/// fn localparse_set(i:&[u8]) -> IResult<&[u8],DerObject> {
+/// fn localparse_set(i:&[u8]) -> IResult<&[u8],BerObject> {
 ///     parse_der_set_defined!(i,
-///         parse_der_integer,
-///         parse_der_integer
+///         parse_ber_integer,
+///         parse_ber_integer
 ///     )
 /// }
 /// let empty = &b""[..];
-/// let bytes = [ 0x30, 0x0a,
+/// let bytes = [ 0x31, 0x0a,
 ///               0x02, 0x03, 0x01, 0x00, 0x01,
 ///               0x02, 0x03, 0x01, 0x00, 0x00,
 /// ];
-/// let expected  = DerObject::from_obj(DerObjectContent::Set(vec![
-///     DerObject::from_int_slice(b"\x01\x00\x01"),
-///     DerObject::from_int_slice(b"\x01\x00\x00"),
-/// ]));
+/// let expected  = BerObject::from_set(vec![
+///     BerObject::from_int_slice(b"\x01\x00\x01"),
+///     BerObject::from_int_slice(b"\x01\x00\x00"),
+/// ]);
 /// assert_eq!(localparse_set(&bytes), Ok((empty, expected)));
 /// # }
 /// ```
@@ -325,8 +325,8 @@ macro_rules! parse_der_set_defined(
     ($i:expr, $($args:tt)*) => (
         map!(
             $i,
-            parse_der_defined!(0x11, $($args)*),
-            |(hdr,o)| $crate::DerObject::from_header_and_content(hdr,$crate::DerObjectContent::Set(o))
+            parse_der_defined!($crate::ber::BerTag::Set, $($args)*),
+            |(hdr,o)| $crate::ber::BerObject::from_header_and_content(hdr,$crate::ber::BerObjectContent::Set(o))
         )
     );
 );
@@ -335,37 +335,37 @@ macro_rules! parse_der_set_defined(
 ///
 /// Given a subparser for a DER type, parse a sequence of identical objects.
 ///
-/// ```rust,no_run
+/// ```rust
 /// # #[macro_use] extern crate nom;
 /// # #[macro_use] extern crate rusticata_macros;
 /// # #[macro_use] extern crate der_parser;
-/// use der_parser::*;
+/// use der_parser::ber::*;
 /// use nom::{IResult,Err,ErrorKind};
 ///
 /// # fn main() {
-/// fn parser(i:&[u8]) -> IResult<&[u8],DerObject> {
-///     parse_der_sequence_of!(i, parse_der_integer)
+/// fn parser(i:&[u8]) -> IResult<&[u8],BerObject> {
+///     parse_der_sequence_of!(i, parse_ber_integer)
 /// };
 /// let empty = &b""[..];
 /// let bytes = [ 0x30, 0x0a,
 ///               0x02, 0x03, 0x01, 0x00, 0x01,
 ///               0x02, 0x03, 0x01, 0x00, 0x00,
 /// ];
-/// let expected  = DerObject::from_obj(DerObjectContent::Sequence(vec![
-///     DerObject::from_int_slice(b"\x01\x00\x01"),
-///     DerObject::from_int_slice(b"\x01\x00\x00"),
-/// ]));
+/// let expected  = BerObject::from_seq(vec![
+///     BerObject::from_int_slice(b"\x01\x00\x01"),
+///     BerObject::from_int_slice(b"\x01\x00\x00"),
+/// ]);
 /// assert_eq!(parser(&bytes), Ok((empty, expected)));
 /// # }
 /// ```
 #[macro_export]
 macro_rules! parse_der_sequence_of(
     ($i:expr, $f:ident) => ({
-        use $crate::der_read_element_header;
+        use $crate::ber::ber_read_element_header;
         do_parse!(
             $i,
-            hdr:     der_read_element_header >>
-                     error_if!(hdr.tag != DerTag::Sequence as u8, ErrorKind::Custom($crate::DER_TAG_ERROR)) >>
+            hdr:     ber_read_element_header >>
+                     error_if!(hdr.tag != $crate::ber::BerTag::Sequence, ErrorKind::Custom($crate::error::BER_TAG_ERROR)) >>
             content: flat_take!(hdr.len as usize,
                 do_parse!(
                     r: many0!(complete!($f)) >>
@@ -373,7 +373,7 @@ macro_rules! parse_der_sequence_of(
                     ( r )
                 )
             ) >>
-            ( $crate::DerObject::from_header_and_content(hdr, $crate::DerObjectContent::Sequence(content)) )
+            ( $crate::ber::BerObject::from_header_and_content(hdr, $crate::ber::BerObjectContent::Sequence(content)) )
         )
     })
 );
@@ -382,37 +382,37 @@ macro_rules! parse_der_sequence_of(
 ///
 /// Given a subparser for a DER type, parse a set of identical objects.
 ///
-/// ```rust,no_run
+/// ```rust
 /// # #[macro_use] extern crate nom;
 /// # #[macro_use] extern crate rusticata_macros;
 /// # #[macro_use] extern crate der_parser;
-/// use der_parser::*;
+/// use der_parser::ber::*;
 /// use nom::{IResult,Err,ErrorKind};
 ///
 /// # fn main() {
-/// fn parser(i:&[u8]) -> IResult<&[u8],DerObject> {
-///     parse_der_set_of!(i, parse_der_integer)
+/// fn parser(i:&[u8]) -> IResult<&[u8],BerObject> {
+///     parse_der_set_of!(i, parse_ber_integer)
 /// };
 /// let empty = &b""[..];
-/// let bytes = [ 0x30, 0x0a,
+/// let bytes = [ 0x31, 0x0a,
 ///               0x02, 0x03, 0x01, 0x00, 0x01,
 ///               0x02, 0x03, 0x01, 0x00, 0x00,
 /// ];
-/// let expected  = DerObject::from_obj(DerObjectContent::Set(vec![
-///     DerObject::from_int_slice(b"\x01\x00\x01"),
-///     DerObject::from_int_slice(b"\x01\x00\x00"),
-/// ]));
+/// let expected  = BerObject::from_set(vec![
+///     BerObject::from_int_slice(b"\x01\x00\x01"),
+///     BerObject::from_int_slice(b"\x01\x00\x00"),
+/// ]);
 /// assert_eq!(parser(&bytes), Ok((empty, expected)));
 /// # }
 /// ```
 #[macro_export]
 macro_rules! parse_der_set_of(
     ($i:expr, $f:ident) => ({
-        use $crate::der_read_element_header;
+        use $crate::ber::ber_read_element_header;
         do_parse!(
             $i,
-            hdr:     der_read_element_header >>
-                     error_if!(hdr.tag != DerTag::Set as u8, ErrorKind::Custom($crate::DER_TAG_ERROR)) >>
+            hdr:     ber_read_element_header >>
+                     error_if!(hdr.tag != $crate::ber::BerTag::Set, ErrorKind::Custom($crate::error::BER_TAG_ERROR)) >>
             content: flat_take!(hdr.len as usize,
                 do_parse!(
                     r: many0!(complete!($f)) >>
@@ -420,7 +420,7 @@ macro_rules! parse_der_set_of(
                     ( r )
                 )
             ) >>
-            ( $crate::DerObject::from_header_and_content(hdr, $crate::DerObjectContent::Set(content)) )
+            ( $crate::ber::BerObject::from_header_and_content(hdr, $crate::ber::BerObjectContent::Set(content)) )
         )
     })
 );
@@ -430,11 +430,11 @@ macro_rules! parse_der_set_of(
 /// Try to parse an optional DER element, and return it as a `ContextSpecific` item with tag 0.
 /// If the parsing failed, the `ContextSpecific` object has value `None`.
 ///
-/// ```rust,no_run
+/// ```rust
 /// # #[macro_use] extern crate nom;
 /// # #[macro_use] extern crate rusticata_macros;
 /// # #[macro_use] extern crate der_parser;
-/// use der_parser::*;
+/// use der_parser::ber::*;
 /// use nom::{IResult,Err,ErrorKind};
 ///
 /// # fn main() {
@@ -444,27 +444,27 @@ macro_rules! parse_der_set_of(
 ///                0x02, 0x03, 0x01, 0x00, 0x01];
 /// let bytes2 = [ 0x30, 0x05,
 ///                0x02, 0x03, 0x01, 0x00, 0x01];
-/// let expected1  = DerObject::from_obj(DerObjectContent::Sequence(vec![
-///     DerObject::from_obj(
-///         DerObjectContent::ContextSpecific(0,
-///             Some(Box::new(DerObject::from_obj(DerObjectContent::Enum(1)))))
+/// let expected1  = BerObject::from_seq(vec![
+///     BerObject::from_obj(
+///         BerObjectContent::ContextSpecific(BerTag(0),
+///             Some(Box::new(BerObject::from_obj(BerObjectContent::Enum(1)))))
 ///     ),
-///     DerObject::from_int_slice(b"\x01\x00\x01"),
-/// ]));
-/// let expected2  = DerObject::from_obj(DerObjectContent::Sequence(vec![
-///     DerObject::from_obj(
-///         DerObjectContent::ContextSpecific(0, None),
+///     BerObject::from_int_slice(b"\x01\x00\x01"),
+/// ]);
+/// let expected2  = BerObject::from_seq(vec![
+///     BerObject::from_obj(
+///         BerObjectContent::ContextSpecific(BerTag(0), None),
 ///     ),
-///     DerObject::from_int_slice(b"\x01\x00\x01"),
-/// ]));
+///     BerObject::from_int_slice(b"\x01\x00\x01"),
+/// ]);
 ///
-/// fn parse_optional_enum(i:&[u8]) -> IResult<&[u8],DerObject> {
-///     parse_der_optional!(i, parse_der_enum)
+/// fn parse_optional_enum(i:&[u8]) -> IResult<&[u8],BerObject> {
+///     parse_der_optional!(i, parse_ber_enum)
 /// }
-/// fn parser(i:&[u8]) -> IResult<&[u8],DerObject> {
+/// fn parser(i:&[u8]) -> IResult<&[u8],BerObject> {
 ///     parse_der_sequence_defined!(i,
 ///         parse_optional_enum,
-///         parse_der_integer
+///         parse_ber_integer
 ///     )
 /// };
 ///
@@ -480,12 +480,12 @@ macro_rules! parse_der_optional(
             do_parse!(
                 content: call!($f) >>
                 (
-                    $crate::DerObject::from_obj(
-                        $crate::DerObjectContent::ContextSpecific(0 /* XXX */,Some(Box::new(content)))
+                    $crate::ber::BerObject::from_obj(
+                        $crate::ber::BerObjectContent::ContextSpecific($crate::ber::BerTag(0) /* XXX */,Some(Box::new(content)))
                     )
                 )
             ) |
-            apply!(parse_der_explicit_failed,0 /* XXX */)
+            apply!($crate::ber::parse_ber_explicit_failed,$crate::ber::BerTag(0) /* XXX */)
         )
     )
 );
@@ -495,7 +495,7 @@ macro_rules! parse_der_optional(
 /// Read a constructed DER element (sequence or set, typically) using the provided functions.
 /// This is generally used to build a struct from a DER sequence.
 ///
-/// The returned object is a tuple containing a [`DerObjectHeader`](struct.DerObjectHeader.html)
+/// The returned object is a tuple containing a [`BerObjectHeader`](struct.BerObjectHeader.html)
 /// and the object returned by the subparser.
 ///
 /// To ensure the subparser consumes all bytes from the constructed object, add the `empty!()`
@@ -512,25 +512,25 @@ macro_rules! parse_der_optional(
 ///
 /// Basic struct parsing (ignoring tag):
 ///
-/// ```rust,no_run
+/// ```rust
 /// # #[macro_use] extern crate nom;
 /// # #[macro_use] extern crate rusticata_macros;
 /// # #[macro_use] extern crate der_parser;
-/// use der_parser::*;
+/// use der_parser::ber::*;
 /// use nom::{IResult,Err,ErrorKind};
 ///
 /// # fn main() {
 /// #[derive(Debug, PartialEq)]
 /// struct MyStruct<'a>{
-///     a: DerObject<'a>,
-///     b: DerObject<'a>,
+///     a: BerObject<'a>,
+///     b: BerObject<'a>,
 /// }
 ///
-/// fn parse_struct01(i: &[u8]) -> IResult<&[u8],(DerObjectHeader,MyStruct)> {
+/// fn parse_struct01(i: &[u8]) -> IResult<&[u8],(BerObjectHeader,MyStruct)> {
 ///     parse_der_struct!(
 ///         i,
-///         a: parse_der_integer >>
-///         b: parse_der_integer >>
+///         a: parse_ber_integer >>
+///         b: parse_ber_integer >>
 ///            empty!() >>
 ///         ( MyStruct{ a: a, b: b } )
 ///     )
@@ -542,15 +542,15 @@ macro_rules! parse_der_optional(
 /// ];
 /// let empty = &b""[..];
 /// let expected = (
-///     DerObjectHeader{
+///     BerObjectHeader{
 ///         class: 0,
 ///         structured: 1,
-///         tag: 0x10,
+///         tag: BerTag::Sequence,
 ///         len: 0xa,
 ///     },
 ///     MyStruct {
-///         a: DerObject::from_int_slice(b"\x01\x00\x01"),
-///         b: DerObject::from_int_slice(b"\x01\x00\x00"),
+///         a: BerObject::from_int_slice(b"\x01\x00\x01"),
+///         b: BerObject::from_int_slice(b"\x01\x00\x00"),
 ///     }
 /// );
 /// let res = parse_struct01(&bytes);
@@ -560,24 +560,24 @@ macro_rules! parse_der_optional(
 ///
 /// To check the expected tag, use the `TAG <tagname>` variant:
 ///
-/// ```rust,no_run
+/// ```rust
 /// # #[macro_use] extern crate nom;
 /// # #[macro_use] extern crate rusticata_macros;
 /// # #[macro_use] extern crate der_parser;
-/// # use der_parser::*;
+/// # use der_parser::ber::*;
 /// # use nom::{IResult,Err,ErrorKind};
 /// # fn main() {
 /// struct MyStruct<'a>{
-///     a: DerObject<'a>,
-///     b: DerObject<'a>,
+///     a: BerObject<'a>,
+///     b: BerObject<'a>,
 /// }
 ///
-/// fn parse_struct_with_tag(i: &[u8]) -> IResult<&[u8],(DerObjectHeader,MyStruct)> {
+/// fn parse_struct_with_tag(i: &[u8]) -> IResult<&[u8],(BerObjectHeader,MyStruct)> {
 ///     parse_der_struct!(
 ///         i,
-///         TAG DerTag::Sequence,
-///         a: parse_der_integer >>
-///         b: parse_der_integer >>
+///         TAG BerTag::Sequence,
+///         a: parse_ber_integer >>
+///         b: parse_ber_integer >>
 ///            empty!() >>
 ///         ( MyStruct{ a: a, b: b } )
 ///     )
@@ -587,20 +587,20 @@ macro_rules! parse_der_optional(
 #[macro_export]
 macro_rules! parse_der_struct(
     ($i:expr, TAG $tag:expr, $($rest:tt)*) => ({
-        use $crate::{DerObjectHeader,der_read_element_header};
+        use $crate::ber::{BerObjectHeader,ber_read_element_header};
         do_parse!(
             $i,
-            hdr: verify!(der_read_element_header, |ref hdr: DerObjectHeader|
-                         hdr.structured == 1 && hdr.tag == $tag as u8) >>
+            hdr: verify!(ber_read_element_header, |ref hdr: BerObjectHeader|
+                         hdr.structured == 1 && hdr.tag == $tag) >>
             res: flat_take!(hdr.len as usize, do_parse!( $($rest)* )) >>
             (hdr,res)
         )
     });
     ($i:expr, $($rest:tt)*) => ({
-        use $crate::{DerObjectHeader,der_read_element_header};
+        use $crate::ber::{BerObjectHeader,ber_read_element_header};
         do_parse!(
             $i,
-            hdr: verify!(der_read_element_header, |ref hdr: DerObjectHeader| hdr.structured == 1) >>
+            hdr: verify!(ber_read_element_header, |ref hdr: BerObjectHeader| hdr.structured == 1) >>
             res: flat_take!(hdr.len as usize, do_parse!( $($rest)* )) >>
             (hdr,res)
         )
@@ -614,7 +614,7 @@ macro_rules! parse_der_struct(
 /// The returned object is either the object returned by the subparser, or a nom error.
 /// Unlike [`parse_der_explicit`](fn.parse_der_explicit.html) or
 /// [`parse_der_implicit`](fn.parse_der_implicit.html), the returned values are *not* encapsulated
-/// in a `DerObject` (they are directly returned, without the tag).
+/// in a `BerObject` (they are directly returned, without the tag).
 ///
 /// To specify the kind of tag, use the EXPLICIT or IMPLICIT keyword. If no keyword is specified,
 /// the parsing is EXPLICIT by default.
@@ -626,19 +626,19 @@ macro_rules! parse_der_struct(
 ///
 /// The following parses `[2] INTEGER`:
 ///
-/// ```rust,no_run
+/// ```rust
 /// # #[macro_use] extern crate nom;
 /// # #[macro_use] extern crate rusticata_macros;
 /// # #[macro_use] extern crate der_parser;
-/// use der_parser::*;
+/// use der_parser::ber::*;
 /// use nom::{IResult,Err,ErrorKind};
 ///
 /// # fn main() {
 /// fn parse_int_explicit(i:&[u8]) -> IResult<&[u8],u32> {
 ///     map_res!(
 ///         i,
-///         parse_der_tagged!(EXPLICIT 2, parse_der_integer),
-///         |x: DerObject| x.as_u32()
+///         parse_der_tagged!(EXPLICIT 2, parse_ber_integer),
+///         |x: BerObject| x.as_u32()
 ///     )
 /// }
 /// let bytes = &[0xa2, 0x05, 0x02, 0x03, 0x01, 0x00, 0x01];
@@ -655,22 +655,22 @@ macro_rules! parse_der_struct(
 ///
 /// The following parses `[2] IMPLICIT INTEGER`:
 ///
-/// ```rust,no_run
+/// ```rust
 /// # #[macro_use] extern crate nom;
 /// # #[macro_use] extern crate rusticata_macros;
 /// # #[macro_use] extern crate der_parser;
-/// use der_parser::*;
+/// use der_parser::ber::*;
 /// use nom::{IResult,Err,ErrorKind};
 ///
 /// # fn main() {
 /// fn parse_int_implicit(i:&[u8]) -> IResult<&[u8],u32> {
 ///     map_res!(
 ///         i,
-///         parse_der_tagged!(IMPLICIT 2, DerTag::Integer),
-///         |x: DerObject| x.as_u32()
+///         parse_der_tagged!(IMPLICIT 2, BerTag::Integer),
+///         |x: BerObject| x.as_u32()
 ///     )
 /// }
-/// let bytes = &[0xa2, 0x03, 0x01, 0x00, 0x01];
+/// let bytes = &[0x82, 0x03, 0x01, 0x00, 0x01];
 /// let res = parse_int_implicit(bytes);
 /// match res {
 ///     Ok((rem,val)) => {
@@ -684,30 +684,30 @@ macro_rules! parse_der_struct(
 #[macro_export]
 macro_rules! parse_der_tagged(
     ($i:expr, EXPLICIT $tag:expr, $f:ident) => ({
-        use $crate::{DerObjectHeader,der_read_element_header};
+        use $crate::ber::{BerObjectHeader,ber_read_element_header};
         do_parse!(
             $i,
-            hdr: verify!(der_read_element_header, |ref hdr: DerObjectHeader| hdr.tag == $tag) >>
+            hdr: verify!(ber_read_element_header, |ref hdr: BerObjectHeader| hdr.tag.0 == $tag) >>
             res: flat_take!(hdr.len as usize, call!( $f )) >>
             (res)
         )
     });
     ($i:expr, EXPLICIT $tag:expr, $submac:ident!( $($args:tt)*)) => ({
-        use $crate::{DerObjectHeader,der_read_element_header};
+        use $crate::ber::{BerObjectHeader,ber_read_element_header};
         do_parse!(
             $i,
-            hdr: verify!(der_read_element_header, |ref hdr: DerObjectHeader| hdr.tag == $tag) >>
+            hdr: verify!(ber_read_element_header, |ref hdr: BerObjectHeader| hdr.tag.0 == $tag) >>
             res: flat_take!(hdr.len as usize, $submac!( $($args)* )) >>
             (res)
         )
     });
     ($i:expr, IMPLICIT $tag:expr, $type:expr) => ({
-        use $crate::{DerObjectHeader,der_read_element_header,der_read_element_content_as};
+        use $crate::ber::{BerObjectHeader,ber_read_element_header,ber_read_element_content_as};
         do_parse!(
             $i,
-            hdr: verify!(der_read_element_header, |ref hdr: DerObjectHeader| hdr.tag == $tag) >>
-            res: call!(der_read_element_content_as, $type as u8, hdr.len as usize) >>
-            (DerObject::from_obj(res))
+            hdr: verify!(ber_read_element_header, |ref hdr: BerObjectHeader| hdr.tag.0 == $tag) >>
+            res: call!(ber_read_element_content_as, $type, hdr.len as usize, hdr.is_constructed(), 0) >>
+            (BerObject::from_obj(res))
         )
     });
     ($i:expr, $tag:expr, $f:ident) => ( parse_der_tagged!($i, EXPLICIT $tag, $f) );
@@ -718,7 +718,7 @@ macro_rules! parse_der_tagged(
 /// Read an application DER element using the provided functions.
 /// This is generally used to build a struct from a DER sequence.
 ///
-/// The returned object is a tuple containing a [`DerObjectHeader`](struct.DerObjectHeader.html)
+/// The returned object is a tuple containing a [`BerObjectHeader`](struct.BerObjectHeader.html)
 /// and the object returned by the subparser.
 ///
 /// To ensure the subparser consumes all bytes from the constructed object, add the `empty!()`
@@ -728,11 +728,11 @@ macro_rules! parse_der_tagged(
 ///
 /// The following parses `[APPLICATION 2] INTEGER`:
 ///
-/// ```rust,no_run
+/// ```rust
 /// # #[macro_use] extern crate nom;
 /// # #[macro_use] extern crate rusticata_macros;
 /// # #[macro_use] extern crate der_parser;
-/// use der_parser::*;
+/// use der_parser::ber::*;
 /// use nom::{IResult,Err,ErrorKind};
 ///
 /// # fn main() {
@@ -740,11 +740,11 @@ macro_rules! parse_der_tagged(
 /// struct SimpleStruct {
 ///     a: u32,
 /// };
-/// fn parse_app01(i:&[u8]) -> IResult<&[u8],(DerObjectHeader,SimpleStruct)> {
+/// fn parse_app01(i:&[u8]) -> IResult<&[u8],(BerObjectHeader,SimpleStruct)> {
 ///     parse_der_application!(
 ///         i,
 ///         APPLICATION 2,
-///         a: map_res!(parse_der_integer,|x: DerObject| x.as_u32()) >>
+///         a: map_res!(parse_ber_integer,|x: BerObject| x.as_u32()) >>
 ///            empty!() >>
 ///         ( SimpleStruct{ a:a } )
 ///     )
@@ -754,7 +754,7 @@ macro_rules! parse_der_tagged(
 /// match res {
 ///     Ok((rem,(hdr,app))) => {
 ///         assert!(rem.is_empty());
-///         assert_eq!(hdr.tag, 2);
+///         assert_eq!(hdr.tag, BerTag::Integer);
 ///         assert!(hdr.is_application());
 ///         assert_eq!(hdr.structured, 1);
 ///         assert_eq!(app, SimpleStruct{ a:0x10001 });
@@ -766,11 +766,11 @@ macro_rules! parse_der_tagged(
 #[macro_export]
 macro_rules! parse_der_application(
     ($i:expr, APPLICATION $tag:expr, $($rest:tt)*) => ({
-        use $crate::{DerObjectHeader,der_read_element_header};
+        use $crate::ber::{BerObjectHeader,ber_read_element_header};
         do_parse!(
             $i,
-            hdr: verify!(der_read_element_header, |ref hdr: DerObjectHeader|
-                         hdr.class == 0b01 && hdr.tag == $tag) >>
+            hdr: verify!(ber_read_element_header, |ref hdr: BerObjectHeader|
+                         hdr.class == 0b01 && hdr.tag.0 == $tag) >>
             res: flat_take!(hdr.len as usize, do_parse!( $($rest)* )) >>
             (hdr,res)
         )
